#!/usr/bin/ruby1.8
#
# Battery observer
#
# --
# Copyright 2005 - Jannis Pohlmann, devel@sten-net.de
# License: GPL

require "gtk2"

class BatteryObserver

   def initialize
       @buffer = Gtk::TextBuffer.new
       
       # Init program
       Gtk.init
   
       # Window
       window = Gtk::Window.new
       window.title = "Battery observer"
       window.border_width = 10
       window.signal_connect("destroy") do Gtk.main_quit end

       # Main frame
       main = Gtk::VBox.new
       window.add(main)
       
       # Buttons frame
       buttons = Gtk::HBox.new
       main.add(buttons)
       
       # Refresh button
       btnstart = Gtk::Button.new("_Start")
       btnstart.signal_connect("clicked") do start(500) end
       buttons.pack_start(btnstart, true, true, 4)
       
       # Stop button
       btnstop = Gtk::Button.new("Sto_p")
       btnstop.signal_connect("clicked") do stop end
       buttons.pack_start(btnstop, true, true, 4)

       # Close button
       btnclose = Gtk::Button.new("_Close")
       btnclose.signal_connect("clicked") do  quit end
       buttons.pack_start(btnclose, true, true, 4)
       
       # Frame
       frame = Gtk::Frame.new
       frame.set_size_request(275, 90)
       main.pack_start(frame, true, true, 8)
       
       # Text widget
       text = Gtk::TextView.new(@buffer)
       text.modify_font(Pango::FontDescription.new("monospace,90"))
       text.editable = false
       text.cursor_visible = false
       frame.add(text)
       
       # Status
       @status = Gtk::Label.new("")
       main.pack_start(@status, true, true, 4)
       
       # Start refreshing
       start(500)
       
       # Run
       window.show_all
       Gtk.main
   end
   
   def quit
       Gtk.main_quit
   end
   
   def start(interval)
       if @timer
           stop
       end
       @timer = Gtk::timeout_add(interval) do refresh end
   end
   
   def stop
       unless @timer.nil?
           Gtk::timeout_remove(@timer)
           @timer = nil
       end
   end
   
   def refresh
       cmd = `cat /proc/acpi/battery/BAT0/state 2>&1`
       if cmd.include? "charged\n"
           stop
           @status.text = "Battery is charged."
           @buffer.text = cmd
           start(2000)
       else
           if cmd.include? "critical\n"
               @status.text = "Discharging. CRITICAL!"
           elsif cmd.include? "discharging\n"
               @status.text = "Discharging."
           else cmd.include? "charging\n"
               @status.text = "Charging."
           end
           @buffer.text = cmd
       end
   end
   
end

BatteryObserver.new
