#!/usr/bin/ruby
#
# Battery observer
#
# --
# Copyright 2005 - Jannis Pohlmann, devel@sten-net.de
# License: GPL

# Configuration
INFO_FILE = "/proc/acpi/battery/BAT0/info"
STATE_FILE = "/proc/acpi/battery/BAT0/state"

# Program
require "gtk2"

class BatteryObserver

       def initialize
               @buffer = Gtk::TextBuffer.new
               
               # Init program
               Gtk.init
       
               # Window
               window = Gtk::Window.new
               window.title = "Battery observer"
               window.border_width = 10
               window.signal_connect("destroy") do Gtk.main_quit end

               # Main frame
               main = Gtk::VBox.new
               window.add(main)
               
               # Buttons frame
               buttons = Gtk::HBox.new
               main.add(buttons)
               
               # Refresh button
               btnstart = Gtk::Button.new("_Start")
               btnstart.signal_connect("clicked") do start(1500); refresh end
               buttons.pack_start(btnstart, true, true, 4)
               
               # Stop button
               btnstop = Gtk::Button.new("Sto_p")
               btnstop.signal_connect("clicked") do stop end
               buttons.pack_start(btnstop, true, true, 4)

               # Close button
               btnclose = Gtk::Button.new("_Close")
               btnclose.signal_connect("clicked") do  quit end
               buttons.pack_start(btnclose, true, true, 4)
               
               # Frame
               frame = Gtk::Frame.new
               frame.set_size_request(275, 90)
               main.pack_start(frame, true, true, 8)
               
               # Text widget
               text = Gtk::TextView.new(@buffer)
               text.modify_font(Pango::FontDescription.new("monospace,90"))
               text.editable = false
               text.cursor_visible = false
               frame.add(text)
               
               # Status
               @status = Gtk::Label.new("")
               main.pack_start(@status, true, true, 4)
               
               # Remaining time
               @time_remaining = Gtk::Label.new("")
               main.pack_start(@time_remaining, true, true, 4)
               
               # Start refreshing
               start(1500)
               refresh
               
               # Run
               window.show_all
               Gtk.main
       end
       
       def quit
               Gtk.main_quit
       end
       
       def start(interval)
               if @timer
                       stop
               end
               @timer = Gtk::timeout_add(interval) do refresh end
       end
       
       def stop
               unless @timer.nil?
                       Gtk::timeout_remove(@timer)
                       @timer = nil
               end
       end
       
       def refresh
               cmd = `cat #{STATE_FILE} 2>&1`
               lines = cmd.split("\n")
               capacity_state = lines[1].gsub(/^[a-z \:]+[         ]+([a-z]+).*$/, "\\1")
               charging_state = lines[2].gsub(/^[a-z \:]+[         ]+([a-z]+).*$/, "\\1")
               present_rate = lines[3].gsub(/^[a-z \:]+[         ]+([0-9]+).*$/, "\\1").to_i
               remaining = lines[4].gsub(/^[a-z \:]+[         ]+([0-9]+).*$/, "\\1").to_i
               voltage = lines[5].gsub(/^[a-z \:]+[         ]+([0-9]+).*$/, "\\1").to_i
               
               return if present_rate == 0

               if charging_state == "discharging"
                       hours = remaining / present_rate
                       minutes = (60 * remaining) / present_rate
                       seconds = ((60 * 60 * remaining) / present_rate) - (minutes * 60)
               
                       @status.text = "Discharging."
                       @time_remaining.text = "Time remaining: "
                       @time_remaining.text += hours.to_s
                       @time_remaining.text += ":"
                       @time_remaining.text += "0" if minutes.to_s.length == 1
                       @time_remaining.text += minutes.to_s
                       @time_remaining.text += ":"
                       @time_remaining.text += "0" if seconds.to_s.length == 1
                       @time_remaining.text += strseconds
                       @time_remaining.text += " hours"
               elsif charging_state == "charging"
                       info = `cat #{INFO_FILE} 2>&1`
                       lines = info.split("\n")
                       last_capacity = lines[2].gsub(/^[a-z \:]+[         ]+([0-9]+).*$/, "\\1").to_i
                       hours = 60 * ((last_capacity - remaining)/present_rate)
                       minutes = (60 * ((last_capacity.to_f - remaining.to_f) / present_rate.to_f)).round
                       seconds = ((60 * 60 * ((last_capacity.to_f - remaining.to_f) / present_rate.to_f)).round) - (minutes * 60)
                       if seconds < 0
                               seconds = 60 - ((minutes * 60) - ((60 * 60 * ((last_capacity.to_f - remaining.to_f) / present_rate.to_f)).round))
                       end
                       
                       @status.text = "Charging."
                       @time_remaining.text = "Time remaining: "
                       @time_remaining.text += hours.to_s
                       @time_remaining.text += ":"
                       @time_remaining.text += "0" if minutes.to_s.length == 1
                       @time_remaining.text += minutes.to_s
                       @time_remaining.text += ":"
                       @time_remaining.text += "0" if seconds.to_s.length == 1
                       @time_remaining.text += seconds.to_s
                       @time_remaining.text += " seconds"
               else
                       @status.text = "Charged."
                       @time_remaining.text = "Getting power from AC."
               end
               
               if capacity_state == "critical"
                       @status.text += " CRITICAL!"
               end
               
               @buffer.text = cmd
       end
       
end

BatteryObserver.new